-- "Mashed Keys" by David Doran
-- This demo uses the relative position of the keys to move a character.

-- array of keyboard rows
--   offset is the x-offset relative to the first row
--   keys is the list of keys in the row (with the key as passed to keypressed)
local rows = {
	{offset=0, keys={"q", "w", "e", "r", "t", "y", "u", "i", "o", "p"}},
	{offset=14, keys={"a", "s", "d", "f", "g", "h", "j", "k", "l"}},
	{offset=42, keys={"z", "x", "c", "v", "b", "n", "m"}}
}

-- key width and height
local keyw = 49
local keyh = 47

-- space between keys
local xpad = 7
local ypad = 7

-- the origin (only used for drawing the keys in this prototype demo)
local ox = 50
local oy = 50

-- hash of the keys currently being pressed
local activekeys = {}
-- hash of the keys to their coordinates
local keymap = {}

-- calculate the position of a key
function key_coords(n_row, n_key)
	local row = rows[n_row]
	local x = ox + row.offset + (n_key-1)*xpad + (n_key-1/2)*(keyw)
	local y = oy + (n_row-1)*ypad + (n_row-1/2)*keyh
	return {x=x, y=y}
end

-- Get the coordinates of the center key "G"
local center = key_coords(2, 5)

-- adjust the speed of "player" movement
local speed = 7
-- player x and y position
local xpos = 400
local ypos = 400
-- used to scale the x and y offsets to calculate speed
local xmax = 245
local ymax = 75

local screenw = love.graphics.getWidth()
local screenh = love.graphics.getHeight()

function love.load()
	keysimg = love.graphics.newImage("keys.png")
	love.graphics.setBackgroundColor(255, 255, 255)
	-- pre-calculate the coordinates of each key
	for i, row in ipairs(rows) do
		for j=1, #row.keys do
			keymap[row.keys[j]] = key_coords(i, j)
		end
	end
end

function love.update()
	local xoff = 0
	local yoff = 0
	-- loop over the keys currently pressed
	for key, active in pairs(activekeys) do
		-- a value of nil means the key is no longer pressed
		if active and keymap[key] then
			-- get the key's coordinates and add up the x/y offsets
			local coords = keymap[key]
			xoff = xoff + (coords.x - center.x)
			yoff = yoff + (coords.y - center.y)
		end
	end
	-- if we're moving
	if xoff ~= 0 or yoff ~= 0 then
		xpos = xpos + (xoff/xmax)*speed
		ypos = ypos + (yoff/ymax)*speed
	end
	if xpos < 0 or xpos > screenw or ypos < 0 or ypos > screenh then
		xpos = 400
		ypos = 400
	end
	print(xoff .. ", " .. yoff)
end

function love.draw()
	love.graphics.setColor(150,150,150)
	love.graphics.line(50, 0, 50, love.graphics.getHeight())
	love.graphics.line(0, 50, love.graphics.getWidth(), 50)

	-- draw the keys
	love.graphics.draw(keysimg, 50, 50)

	for i, row in ipairs(rows) do
		for j=1, #row.keys do
			local coords = key_coords(i, j)
			-- highlight keys that are pressed
			if activekeys[rows[i].keys[j]] then
				love.graphics.setColor(255, 0, 0)
				-- show the direction from the center to a pressed key
				love.graphics.line(center.x, center.y, coords.x, coords.y)
				-- highlight the pressed key
				love.graphics.circle("line", coords.x, coords.y, 10, 50)
			end
			-- love.graphics.setColor(0, 0, 255)
			-- love.graphics.circle("line", coords.x, coords.y, 10, 50)
		end
	end

	--love.graphics.setColor(0, 255, 0)
	--love.graphics.circle("fill", center.x, center.y, 10, 50)

	love.graphics.setColor(0, 0, 0)
	-- draw the "player's" position
	love.graphics.circle("fill", xpos, ypos, 10, 50)
	love.graphics.print("Press the letter keys to move the ball.\nYou can press more than one at once!", 500, 350)
end

function love.keypressed(key)
	if (key == "escape") then
		love.event.quit()
	else
		activekeys[key] = 1
	end
end

function love.keyreleased(key)
	activekeys[key] = nil
end